<?php
// wcf imports
require_once(WCF_DIR.'lib/data/DatabaseObject.class.php');

// wbb imports
require_once(WBB_DIR.'lib/data/board/Board.class.php');
require_once(WBB_DIR.'lib/data/thread/LatestActiveThreadList.class.php');

/**
 * Represents a LatestPostsAndThreadsList.
 *
 * @author 	Nerd Almighty
 * @copyright	2009-2010 Bergwerk <http://bergwerk.forenberg.de>
 * @license	Creative Commons - Attribution-Noncommercial-Share Alike 3.0 Unported <http://creativecommons.org/licenses/by-nc-sa/3.0/>
 * @package	com.bergwerk.wbb.page.user.latestPostsAndThreads
 * @subpackage	data.thread
 * @category 	Burning Board
 */
class LatestPostsAndThreadsList extends DatabaseObject {
	// threadList parameters
	public $threadListLimit = 20;
	public $threadListOffset = 0;
		
	/**
	 * ThreadList object.
	 *
	 * @var ThreadList
	 */
	public $threadList = null;
	
	/**
	 * Holds a list of parameters of this list.
	 *
	 * @var array<integer>
	 */
	protected $options = null;
	
	/**
	 * Holds the board ids of all boards that can be accessed by the active user.
	 *
	 * @var array<integer>
	 */
	protected $cachedBoardIDs = null;
	
	/**
	 * Holds cached latest posts and threads lists.
	 *
	 * @var array<LatestPostsAndThreadsList>
	 */
	protected static $lists = null;
	
	/**
	 * Holds all board objects that are referenced by threads in the threadList
	 *
	 * @var array<Board>
	 */
	public $boards = array();
	
	/**
	 * Creates a new LatestPostsAndThreadsList object.
	 *
	 * If id is set, the function reads the LatestPostsAndThreadsList data from database.
	 * Otherwise it uses the given resultset.
	 * 
	 * @param 	integer		$listID			id of a list 
	 * @param 	array		$row			resultset with list data form database
	 */
	public function __construct($listID, $row = null) {
		if ($listID) {
			// get data from cache
			$list = self::getList($listID);
			$row = $list->data;
		}
		
		parent::__construct($row);
	}
	
	/**
	 * Checks whether the active user can access this list.
	 *
	 * @param	boolean		$canBeEntered		if true, this function only returns true if the active user is also allowed to enter this list
	 * @return	boolean		$result			true, if this list is enabled, the user has the right to view these lists and the list contains accessible boards
	 */
	public function canBeAccessed($canBeEntered = false) {
		// check group permissions
		if (!WCF::getUser()->getPermission('user.board.canViewLatestPostsAndThreadsList') || ($canBeEntered && !WCF::getUser()->getPermission('user.board.canEnterLatestPostsAndThreadsList'))) return false; 
		// check list state
		if (!$this->isEnabled) return false;
		// check accessible boards
		if (!count($this->getIncludedBoards())) return false;
		
		return true;
	}
	
	/**
	 * Enters the active user to this list.
	 */
	public function enter() {
		if (!$this->listID) {
			throw new IllegalLinkException();
		}
		// check whether this list holds any boards that are accessible by the user
		if (!$this->canBeAccessed(true)) {
			throw new PermissionDeniedException();
		}
		
		// change style if necessary
		require_once(WCF_DIR.'lib/system/style/StyleManager.class.php');
		if ($this->styleID && (!WCF::getSession()->getStyleID() || $this->enforceStyle) && StyleManager::getStyle()->styleID != $this->styleID) {
			StyleManager::changeStyle($this->styleID, true);
		}
	}
	
	/**
	 * Initialises the threadList of this list, if necessary.
	 *
	 * @param	integer		$presortBoardIDs	if positive (> 0), the number of board ids used in the sql query will be reduced to $reduceBoardIDs, before reading threads from database; this option can only be used if only the latest X threads need to be read, without using any kind of offset
	 */
	public function initThreadList($presortBoardIDs = 0) {
		if ($this->threadList === null) {
			require_once(WBB_DIR.'lib/data/thread/LatestActiveThreadList.class.php');
			$this->threadList = new LatestActiveThreadList($this->getIncludedBoards(true), true, $this->getOption('daysPrune'), $presortBoardIDs);
		}
	}
	
	/**
	 * Wrapper for ThreadList->readThreads()
	 * @see ThreadList::readThreads()
	 */
	public function readThreads() {
		$this->initThreadList();
		$this->threadList->limit = $this->threadListLimit;
		$this->threadList->offset = $this->threadListOffset;
		$this->threadList->readThreads();
		$this->readBoards();
	}
	
	/**
	 * Reads the boards referenced by threads in the threadList.
	 */
	public function readBoards() {
		foreach ($this->threadList->threads as $thread) {
			if (!isset($this->boards[$thread->boardID])) {
				$board = new Board($thread->boardID);
				$this->boards[$thread->boardID] = $board;
			}
		}
	}
	
	/**
	 * Returns either an array or string of board ids that are included in this list.
	 *
	 * @param	boolean		$returnAsString		should be true, to return the list of board ids as a string (seperated by commas)
	 * @return	mixed					a list of board ids included in this list
	 */
	public function getIncludedBoards($returnAsString = false) {
		$result = array();
		
		// get all board ids
		if ($this->cachedBoardIDs === null) {
			$boardIDs = array();
			$includedBoards = array();
			$includeAllBoards = $this->includeAllBoards;
			$excludedBoards = array();
			$excludeIgnoredBoards = (boolean) WCF::getUser()->userID;

			// get in- and excluded boards
			if (!$this->isDefault || !WCF::getUser()->userID || !WCF::getUser()->getPermission('user.latestPostsAndThreads.canSelectBoardsForDefaultList')) {
				$excludedBoards = ($this->includeAllBoards && strlen($this->excludedBoards) ? explode(',', $this->excludedBoards) : array());
				$includedBoards = (!$this->includeAllBoards && strlen($this->includedBoards) ? explode(',', $this->includedBoards) : array());
			}
			else {
				// get user options
				$userIncludedBoardsOption = str_replace("\n", ',', WCF::getUser()->getUserOption('latestPostsAndThreadsDefaultListIncludedBoards'));
				$userIncludeAllBoardsOption = WCF::getUser()->getUserOption('latestPostsAndThreadsDefaultListIncludeAllBoards');
				$userExcludedBoardsOption = str_replace("\n", ',', WCF::getUser()->getUserOption('latestPostsAndThreadsDefaultListExcludedBoards'));
				
				// evaluate list and user options
				$listExcludedBoards = ($this->includeAllBoards && strlen($this->excludedBoards) ? explode(',', $this->excludedBoards) : array());
				$userExcludedBoards = ((!strlen($userIncludedBoardsOption) || $userIncludeAllBoardsOption) && strlen($userExcludedBoardsOption) ? explode(',', $userExcludedBoardsOption) : array());
				$appendUserExcludedBoards = (!strlen($userIncludedBoardsOption) && !$userIncludeAllBoardsOption);
				$excludedBoards = ($appendUserExcludedBoards ? array_merge($listExcludedBoards, $userExcludedBoards) : $userExcludedBoards);
				$includeAllBoards = ($userIncludeAllBoardsOption || ($this->includeAllBoards && !strlen($userIncludedBoardsOption)));
				
				if (!$includeAllBoards) {
					$listIncludedBoards = (strlen($this->includedBoards) ? explode(',', $this->includedBoards) : array());
					$userIncludedBoards = (strlen($userIncludedBoardsOption) ? explode(',', $userIncludedBoardsOption) : array());
					$includedBoards = (count($userIncludedBoards) ? $userIncludedBoards : $listIncludedBoards);
				}
			}	
			
			// get accessible boards
			$boardSelect = array_keys(Board::getBoardSelect(array('canViewBoard', 'canEnterBoard', 'canReadThread'), true, true, $excludedBoards));
			$boardIDs = (count($boardSelect) ? array_combine($boardSelect, $boardSelect) : array());
			
			// remove ignored boards
			if ($excludeIgnoredBoards) {
				$user = WCF::getUser();
				foreach ($boardIDs as $boardID) {
					if ($user->isIgnoredBoard($boardID)) unset($boardIDs[$boardID]);
				}
			}
			
			if (!$includeAllBoards) {
				// check which of the included boards exist and are accesible
				$result = array_intersect($boardIDs, $includedBoards);
			}
			else {
				$result = $boardIDs;
			}
			
			// store board ids
			$this->cachedBoardIDs = $result;
		}
		else {
			// get board ids from cache
			$result = $this->cachedBoardIDs;
		}
		
		return ($returnAsString ? implode(',', $result) : $result);
	}
	
	/**
	 * Returns the value of a list parameter.
	 *
	 * @param	string		$optionName		name of the requested parameter
	 * @param	boolean		$index			should be true, if this list is to be displayed on the index page
	 * @return	mixed		$result			value of the requested paramter
	 */
	public function getOption($optionName, $index = false) {
		if ($this->options === null) {
			// fill array
			$this->options = array(
				'length' => $this->getLength($index),
				'design' => $this->getDesign($index),
				'daysPrune' => $this->getDaysPrune(),
				'showStatusDisplayIcons' => ($this->showStatusDisplayIcons > -1 ? $this->showStatusDisplayIcons : LATEST_POSTS_AND_THREADS_LIST_SHOW_STATUS_DISPLAY_ICONS),
				'showColumnRating' => (THREAD_ENABLE_RATING ? ($this->showColumnRating > -1 ? $this->showColumnRating : LATEST_POSTS_AND_THREADS_LIST_SHOW_COLUMN_RATING) : 0),
				'showColumnReplies' => ($this->showColumnReplies > -1 ? $this->showColumnReplies : LATEST_POSTS_AND_THREADS_LIST_SHOW_COLUMN_REPLIES),
				'showColumnViews' => ($this->showColumnViews > -1 ? $this->showColumnViews : LATEST_POSTS_AND_THREADS_LIST_SHOW_COLUMN_VIEWS),
				'showLastPostTimeIcon' => ($this->showLastPostTimeIcon > -1 ? $this->showLastPostTimeIcon : LATEST_POSTS_AND_THREADS_LIST_SHOW_LAST_POST_TIME_ICON)
			);
		}
		
		return ($this->options[$optionName]);
	}
	
	/**
	 * Returns the number of threads that should be shown on the index / board page.
	 *
	 * @param	boolean		$index			should be true, if this list is to be displayed on the index page
	 * @return	integer		$length			number of threads
	 */
	protected function getLength($index = false) {
		// default value
		$length = LATEST_POSTS_AND_THREADS_LIST_LENGTH;
		
		// list option
		if ($this->length) $length = $this->length;
		
		// index option
		if ($index && LATEST_POSTS_AND_THREADS_INDEX_LIST_LENGTH) $length = LATEST_POSTS_AND_THREADS_INDEX_LIST_LENGTH;
		
		// user options
		if (WCF::getUser()->userID && WCF::getUser()->getPermission('user.latestPostsAndThreads.canCustomizeList')) {
			if (WCF::getUser()->getUserOption('latestPostsAndThreadsListLength')) $length = WCF::getUser()->getUserOption('latestPostsAndThreadsListLength');
			if ($index && WCF::getUser()->getPermission('user.latestPostsAndThreads.canCustomizeIndexList') && WCF::getUser()->getUserOption('latestPostsAndThreadsIndexListLength')) $length = WCF::getUser()->getUserOption('latestPostsAndThreadsIndexListLength');
		}
		
		return $length;
	}
	
	/**
	 * Returns the design that should be used for displaying the latest posts and threads box on the index / board page.
	 *
	 * @param	boolean		$index			should be true, if this list is to be displayed on the index page
	 * @return	integer		$design			1 = short, 2 = detailed
	 */
	protected function getDesign($index = false) {
		// default value
		$design = LATEST_POSTS_AND_THREADS_LIST_DESIGN;
		
		// list option
		if ($this->design) $design = $this->design;
		
		// index option
		if ($index && LATEST_POSTS_AND_THREADS_INDEX_LIST_DESIGN) $design = LATEST_POSTS_AND_THREADS_INDEX_LIST_DESIGN;
		
		// user options
		if (WCF::getUser()->userID && WCF::getUser()->getPermission('user.latestPostsAndThreads.canCustomizeList')) {
			if (WCF::getUser()->getUserOption('latestPostsAndThreadsListDesign')) $design = WCF::getUser()->getUserOption('latestPostsAndThreadsListDesign');
			if ($index && WCF::getUser()->getPermission('user.latestPostsAndThreads.canCustomizeIndexList') && WCF::getUser()->getUserOption('latestPostsAndThreadsIndexListDesign')) $design = WCF::getUser()->getUserOption('latestPostsAndThreadsIndexListDesign');
		}
		
		return $design;
	}
	
	/**
	 * Returns the maximum age of posts and threads in this list; any older posts will not be included.
	 *
	 * @return	integer		$daysPrune		number of days
	 */
	protected function getDaysPrune() {
		// default value
		$daysPrune = LATEST_POSTS_AND_THREADS_LIST_DAYS_PRUNE;
		
		// list option
		if ($this->daysPrune) $daysPrune = $this->daysPrune;
		
		// user option
		if (WCF::getUser()->userID && WCF::getUser()->getPermission('user.latestPostsAndThreads.canCustomizeList')) {
			if (WCF::getUser()->getUserOption('latestPostsAndThreadsListDaysPrune')) $daysPrune = WCF::getUser()->getUserOption('latestPostsAndThreadsListDaysPrune');
		}
		
		return $daysPrune;
	}
	
	/**
	 * Gets the list with the given list id from cache.
	 *
	 * @param	integer				$listID		id of the requested list
	 * @return	LatestPostsAndThreadsList
	 */
	public static function getList($listID) {
		if (self::$lists === null) {
			// add cache resource
			self::addCacheResource();
			// get from cache
			self::$lists = WCF::getCache()->get('latestPostsAndThreadsList', 'lists');
		}
		
		if (!isset(self::$lists[$listID])) return new LatestPostsAndThreadsList(null, null);
		
		return self::$lists[$listID];
	}
	
	/**
	 * Returns the default latest posts and threads list.
	 *
	 * @return	LatestPostsAndThreadsList	the default latest posts and threads list
	 */
	public static function getDefaultList() {
		// get data from cache
		self::addCacheResource();
		$defaultList = WCF::getCache()->get('latestPostsAndThreadsList', 'defaultList');
		
		return $defaultList;
	}
	
	/**
	 * Returns the list connected to the given board id, or false if no such list exists.
	 *
	 * @param	integer		$boardID	id of a board
	 * @return	mixed				the list connected to the given board id or false, if no such list exists
	 */
	public static function getListByBoardID($boardID) {
		// get data from cache
		self::addCacheResource();
		$boardIDsToListID = WCF::getCache()->get('latestPostsAndThreadsList', 'boardIDsToListID');
		
		if (!isset($boardIDsToListID[$boardID])) return false;
		
		return self::getList($boardIDsToListID[$boardID]);
	}
	
	/**
	 * Adds the list cache resource.
	 */
	public static function addCacheResource() {
		WCF::getCache()->addResource('latestPostsAndThreadsList', WBB_DIR.'cache/cache.latestPostsAndThreadsList.php', WBB_DIR.'lib/system/cache/CacheBuilderLatestPostsAndThreadsList.class.php');
	}
	
	/**
	 * Resets the list cache.
	 */
	public static function resetCache() {
		self::addCacheResource();
		WCF::getCache()->clearResource('latestPostsAndThreadsList');
		self::$lists = null;
	}
} 
?>